:- ensure_loaded( 'wn_tools.pl').
:- ensure_loaded('wn_s.pl').
:- ensure_loaded('wn_g.pl').
:- ensure_loaded('wn_hyp.pl').
:- ensure_loaded('wn_mm.pl').
:- ensure_loaded('wn_sim.pl').

%%%%%%%%%%%%%%%%%%%%%%%%%%% Examples %%%%%%%%%%%%%%%%%%%%%%%%%%%

txt1("I once said to a professional Bass, Technique is more for the singer than for the audience. A singer who is confident and excited when walking out on stage can then concentrate on interpretation of the role or aria. The audience automatically benefits from a secure performance.").
txt2("Bass fishing is one of the most popular participation sports in America. It has evolved from fishing for food into a competitive sport in which, ironically, the caught fish are returned to the water in a catch and release program. It has even given its name to the must-have toy for big boys on the East Coast, the Bass Boat.").
txt3("Children are being stripped of their natural creativity by structured activities and hi-tech toys, leading academics warned yesterday. Innate play skills are lost as parents pay for them to attend classes and clubs or buy televisions and video game machines for their bedrooms. Even at school they are told what games to play in sports lessons and sometimes even in the playground. But the regimentation of their leisure time is stifliing their initiative, says a report.").
txt4("Last christmas when I got my guitar, my dad said Id have to play a whole song by the time Id had it for a year. Well, this month I was able to play a whole song on stage, two actually, one with a whole band, so I guess I have learned how to play an instrument. But Ill never stop learning.").
txt5("What follows, is a romantic tale of the river bank. It is a cliche, but meeting someone really does always happen when you least expect it. For me, romance began in a corner of the Fort St George in Cambridge one summers evening when it turned out the guy I liked was also keen on me. It took a second date before we actually got together. What followed was an idyllic summer spent sitting by the river in the Fort St George, watching the boats go by, and dreaming together."). 
txt6("Edvin Libohva, general manager of the Savings Bank, Wednesday traveled to Fier to meet with bank workers overwhelmed by the hundreds of customers lining up outside the bank to withdraw their money. He also reiterated that money deposited in the bank was safe. A day earlier, he warned bank customers that their actions put the bank at risk.").
txt7("Many areas of the brain are insensitive to the lesion making effects of the toxins currently available so this approach has to be used quite selectively a final problem is that these substances may be taken up into the blood supply as it passes through the brain and carried to other parts of the body where they may have toxic effects that confound their effects on the nervous system conceptual problems despite these technical limitations physiological psychologists take the lesion method very much for granted if one were to suggest to an electronics engineer that he might want to study how a circuit works by pulling bits out and observing changes in function his reaction would probably be one of horror his argument would be that most electronic circuits are organized interactively by which we mean that the proper operation of one component depends on the normal operation of all of the others.").
txt8("Word sense disambiguation involves classifying an occurrence of a word in its context, into one of its many discrete senses. Each such instance, composed of the word and its context, can be represented by a rich set of discrete features which may be effectively utilized by decision trees. Pedersen [55] shows that varying the basic learning algorithm yields little variations in the achieved accuracies for a given task of word sense disambiguation. He states that identifying the most useful features to use for disambiguation is of greater significance. He believes that decision tree learning for word sense disambiguation is reasonably accurate and has further advantage of showing relationship amongst features via the structure of the decision tree.").
txt9("Specifies if a new program will be created when there is an existing object program of the same name in the same library specifies that an object program is generated to run with the current release or the previous release of the os operating system the profile of either the program owner or the program user is used to run the program and control which objects can be used by the program including what authority the program has for each object specify the usrprf parameter to reflect the security requirements of your installation specifies the authority granted to the public for the program and its description.").
txt10("In science fiction, the Three Laws of Robotics are a set of three rules written by Isaac Asimov, which almost all positronic robots appearing in his fiction must obey. Introduced in his 1942 short story Runaround, although foreshadowed in a few earlier stories, the Laws state the following: A robot may not injure a human being or, through inaction, allow a human being to come to harm. A robot must obey any orders given to it by human beings, except where such orders would conflict with the First Law. A robot must protect its own existence as long as such protection does not conflict with the First or Second Law.").

ex1(G) :-
     L = [bass, excited, performance, professional],
     T = [professional, bass, technique, singer, audience, confident, excited, stage, concentrate, interpretation, aria, performance],
     disambigua(L, T, G).
ex2(G) :-
     L = [bass, returned, release],
     txt2(T),
     print_info(L, T),
     disambigua(L, T, G).
ex3(G) :-
     L = [play, report, school],
     txt3(T),
     print_info(L, T),
     disambigua(L, T, G).
ex4(G) :-
     L = [play, stage, band],
     txt4(T),
     print_info(L, T),
     disambigua(L, T, G).
ex5(G) :-
     L = [bank, meeting],
     txt5(T),
     print_info(L, T),
     disambigua(L, T, G).
ex6(G) :-
     L = [bank, withdraw, safe],
     txt6(T),
     print_info(L, T),
     disambigua(L, T, G).
ex7(G) :-
     L = [brain, toxins, nervous, circuits, function],
     txt7(T),
     print_info(L, T),
     disambigua(L, T, G).
ex8(G) :-
     L = [sense, tree, word],
     txt8(T),
     print_info(L, T),
     disambigua(L, T, G).
ex9(G) :-
     L = [run, parameter, program],
     txt9(T),
     print_info(L, T),
     disambigua(L, T, G).
ex10(G) :-
     L = [machine, fiction, positronic],
     txt10(T),
     print_info(L, T),
     disambigua(L, T, G).

print_info(List, Text) :-
     write('-- Disambiguating '),
     print_list(List),
     write(' in text "'),
     atom_codes(A, Text),
     write(A),
     write('"...'),
     nl.



%%%%%%%%%%%%%%%%%%%%%%%%%% Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%

min_length(4).                 %4
wg_into_cg_weight(1).          %1
c_into_wg_weight(50).          %50
wg_into_t_weight(25).          %25


%%%%%%%%%%%%%%%%%%%%%%%%%% Predicates %%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%% count_occurrences(+Word, +List, ?Count)
%%%%%%%%% Unifica la variabile Count con il numero di occorrenze dell'atomo Word nella lista List.
count_occurrences(Word, [Word|Tail], Count) :-          %Word  il primo elemento di List
     !,                                                 %taglia
     count_occurrences(Word, Tail, C),                  %conta le occorrenze nella coda
     Count is C + 1.                                    %incrementa il risultato
count_occurrences(Word, [_|Tail], Count) :-             %Word non  il primo elemento di List
     !,
     count_occurrences(Word, Tail, Count).              %conta le occorrenze nella coda
count_occurrences(_, [], 0).


%%%%%%%%% disambigua(+Words, +ContextList, ?Senses)
%%%%%%%%% Unifica la variabile Senses con la lista dei sensi delle parole della lista Words nella lista ContextList.
disambigua([Head|Tail], Contesto, Senses) :-
     findall(Parola,(member(Parola,Contesto),Parola \== Head),ContextList),                                       %la parola da disambiguare non deve fare parte del contesto
     findall(sense(Syn, Head, POS, Value), disambiguate_word(Head, ContextList, POS, Syn, Value), [L1|_]),        %disambigua la parola nel testo
     disambigua(Tail, Contesto, L2),                                                                              %va avanti
     append([L1], L2, Senses).                                                                                    %concatena i risultati
disambigua([], _, []).


%%%%%%%%% disambiguate_word(+Word, +Text, ?Gloss, ?PartOfSpeech, ?Synset) :-
%%%%%%%%% Unifica la variabile Gloss con la definizione del senso per la parola Word nella stringa Text e la variabile PartOfSpeech con la relativa parte del discorso, Synset con il rispettivo Synset, Value con il MaxScore.
disambiguate_word(Word, _, none, none, 0) :-
     not(my_s(_,_,Word,_,_,_)),                                                                 %la parola non  presente in WordNet
     !.                                                                                         %taglia
disambiguate_word(Word, ContextList, PartOfSpeech, Synset, Value) :-                            %la parola  presente in WordNet
     get_glosses_from_context(ContextList, L),                                                  %calcola le definizioni delle parole del contesto
     min_length(Min),                                                                           %recupera il settaggio della lunghezza minima
     filter_min_length(L, Min, [Word], C),                                                      %filtra le definizioni per lunghezza minima delle parole con eccezione della parola da disambiguare
     findall(score(S, V), (my_s(S, _, Word, _, _, _), get_glosses_from_s(S, G),
          filter_min_length(G, Min, [Word], F), score(F, ContextList, C, ContextList, V)), M),  %calcola la lista dei sensi associati ad un valore di valutazione
     max_score(M, Synset, Value),                                                               %calcola il senso con valutazione massima
     part_of_speech(Synset, PartOfSpeech).                                                      %recupera l'identificativo della parte del discorso associata al senso


%%%%%%%%% filter_min_length(+List, +Min, +ExcList, ?Filtered) :-
%%%%%%%%% Unifica la variabile Filtered con la lista delle parole della lista List di lunghezza maggiore o uguale a Min con le eccezioni della lista ExcList.
filter_min_length([Head|Tail], Min, ExcList, [Head|List]) :-
     atom_length(Head, N),                                 %calcola la lunghezza dell'atomo
     N >= Min,                                             %la lunghezza  maggiore o uguale a Min
     !,                                                    %taglia
     filter_min_length(Tail, Min, ExcList, List).          %va avanti
filter_min_length([Head|Tail], Min, ExcList, [Head|List]) :-
     member(Head, ExcList),                                % un'eccezione
     !,                                                    %taglia
     filter_min_length(Tail, Min, ExcList, List).          %va avanti
filter_min_length([_|Tail], Min, ExcList, List) :-
     !,
     filter_min_length(Tail, Min, ExcList, List).          %nessuno dei due casi, non include l'atomo e va avanti
filter_min_length([], _, _, []).

%%%%%%%%% get_glosses_from_context(+Context, ?List)
%%%%%%%%% Unifica la variabile List con la concatenazione delle definizioni dei sensi delle parole contenute nella lista Context.
get_glosses_from_context([], []):-!.
get_glosses_from_context([Head|Tail], List) :-
     get_glosses_from_w(Head, G),                 %trova le definizioni della parola
     get_glosses_from_context(Tail, L1),          %calcola le definizioni della coda
     append(G, L1, List).                         %concatena la definizione con quelle della coda


%%%%%%%%% get_glosses_from_s(+Synset, ?Glosses)
%%%%%%%%% Unifica la variabie Glosses con la concatenazione delle definizioni del synset Synset e delle definizioni dei synset in relazione ad esso.
get_glosses_from_s(Synset, Glosses) :-
     g(Synset, G),                                                                                 %recupera la definizione del synset
     atom_codes(G, L),                                                                             %trasforma la definizione in una lista di caratteri
     tokenize(L, LG),                                                                              %trasforma la lista di caratteri in una lista di parole
     findall(T1, (hyp(Synset, H1), g(H1, G1), atom_codes(G1, A1), tokenize(A1, T1)), L1),          %recupera le definizioni degli iperonimi
     findall(T2, (hyp(H2, Synset), g(H2, G2), atom_codes(G2, A2), tokenize(A2, T2)), L2),          %recupera le definizioni degli iponimi
     findall(T3, (mm(Synset, M1), g(M1, G3), atom_codes(G3, A3), tokenize(A3, T3)), L3),           %recupera le definizioni degli olonimi
     findall(T4, (mm(M2, Synset), g(M2, G4), atom_codes(G4, A4), tokenize(A4, T4)), L4),           %recupera le definizioni degli meronimi
     findall(T5, (sim(Synset, S1), g(S1, G5), atom_codes(G5, A5), tokenize(A5, T5)), L5),          %recupera le definizioni dei simili
     flatten([LG,L1,L2,L3,L4,L5], Glosses).                                                        %concatena tutte le definizioni                                                          


%%%%%%%%% get_glosses_from_w(+Word, ?Glosses)
%%%%%%%%% Unifica la variabile Glosses con la concatenazione dell'insieme delle definizioni per la parola Word.
get_glosses_from_w(Word, Glosses) :-
     findall(G, (my_s(S,_,Word,_,_,_), get_glosses_from_s(S, G)), L),       %recupera le definizioni della parola
     flatten(L, Glosses).                                                   %appiattisce la lista


%%%%%%%%% max_score(+List, ?Synset, ?Value)
%%%%%%%%% Unifica la variabile Value con il valore massimo della lista di risultati List e Synset con il synset associato.
max_score(List, Synset, Value) :-
     max_score_(0, 0, List, Synset, Value).                                         %predicato ausiliario
max_score_(OldSynset, OldValue, [score(_, Value)], OldSynset, OldValue) :-          %ultimo elemento
     OldValue > Value,                                                              %il valore precedente  maggiore di quello dell'elemento attuale della lista
     !.                                                                             %taglia, mantiene il valore precedente
max_score_(_, OldValue, [score(Synset, Value)], Synset, Value) :-                   %ultimo elemento
     OldValue =< Value,                                                             %il valore precedente  minore o uguale a quello dell'elemento attuale della lista
     !.                                                                             %taglia, accetta il valore attuale
max_score_(OldSynset, OldValue, [score(_, Value)|Tail], Synset2, Value2) :-         %pi elementi nella lista
     OldValue > Value,                                                              %il valore precedente  maggiore di quello dell'elemento attuale della lista
     max_score_(OldSynset, OldValue, Tail, Synset2, Value2),                        %va avanti con il valore precedente
     !.                                                                             %taglia
max_score_(_, OldValue, [score(Synset, Value)|Tail], Synset2, Value2) :-            %pi elementi nella lista
     OldValue =< Value,                                                             %il valore precedente  minore o uguale a quello dell'elemento attuale della lista
     max_score_(Synset, Value, Tail, Synset2, Value2).                              %va avanti con il valore attuale
     
%%%%%%%%% part_of_speech(+Synset, ?Pos)
%%%%%%%%% Unifica la variabile Pos con l'indicazione della parte del discorso relativa al synset Synset.
part_of_speech(Synset, noun) :-
     number_codes(Synset, [H|_]),          %recupera il primo carattere del numero di synset
     1 is H - "0",                         % un 1
     !.                                    %taglia,  un sostantivo
part_of_speech(Synset, verb) :-
     number_codes(Synset, [H|_]),          %recupera il primo carattere del numero di synset
     2 is H - "0",                         % un 2
     !.                                    %taglia,  un verbo
part_of_speech(Synset, adjective) :-
     number_codes(Synset, [H|_]),          %recupera il primo carattere del numero di synset
     3 is H - "0",                         % un 3
     !.                                    %taglia,  un aggettivo
part_of_speech(Synset, adverb) :-
     number_codes(Synset, [H|_]),          %recupera il primo carattere del numero di synset
     4 is H - "0",                         % un 4
     !.                                    %taglia,  un avverbio


%%%%%%%%% score(+WordGloss, +Context, +ContextGlosses, +Text, ?Value)
%%%%%%%%% Calcola in Value il valore di bont di un senso a partire dalla sua definizione, il suo contesto, le definizioni del contesto e il testo.
score(WordGloss, Context, ContextGlosses, Text, Value) :-
     score_(WordGloss, ContextGlosses, V1),          %conta quante parole della definizione della parola da disambiguare stanno nelle definizioni delle parole del contesto
     score_(Context, WordGloss, V2),                 %conta quante parole del contesto stanno nella definizione della parola da disambiguare
     score_(WordGloss, Text, V3),                    %conta quante parole della definizione della parola da disambiguare stanno nel testo
     wg_into_cg_weight(F1),                          %recupera il settaggio del primo coefficiente
     c_into_wg_weight(F2),                           %recupera il settaggio del secondo coefficiente
     wg_into_t_weight(F3),                           %recupera il settaggio del terzo coefficiente
     Value is V1*F1 + V2*F2 + V3*F3.                 %calcola il valore finale
score_([], _, 0):-!.
score_([Head|Tail], List, Value) :- 
     score_(Tail, List, V1),                         %conta le occorrenze degli atomi di Tail nella lista List
     count_occurrences(Head, List, V2),              %conta le occorrenze del primo elemento nella lista List
     Value is V1 + V2.                               %somma i due valori

%%%%%%%% nth(?Integer, ?List, ?Term).
nth(1, [X|_], X):-!.
nth(N, [_|C],X):-
     nth(N1,C,X),
     N is N1 + 1.