% Author: Michele Bozzano
% Date: 19/01/2010

%un aggettivo caratterizza un nome
%sense(Syn,Word,POS,Value)
%findRelationsByPOS(+SenseList,?Relations).
%Relations  la lista delle relazioni trovate tra le parole presenti in SenseList, ricavate sulla base della Part of Speech
findRelationsByPOS(SenseList,Relations):-
    numeraLista(SenseList,ListaNumerata),             %assegna un numero progresssivo a ogni elemento della lista e scarta info non necessarie
    bindAdjectives(ListaNumerata,CoppieAN),           %CoppieAN  la lista delle coppie (Aggettivo,Nome di riferimento)
    bindAdverbs(ListaNumerata,CoppieAV),              %CoppieAV  la lista delle coppie (Avverbio,Verbo di riferimento)
    createAssociations(ListaNumerata,Associations),   %Associations  la lista dei gruppi di elementi in relazione tra loro
    append(CoppieAN,CoppieAV,L1),                     %concatena i risultati
    append(L1,Associations,Relations).                %concatena i risultati
    
%numeraLista(+SenseList,?L).
%L  la lista delle triple (Word,Pos,N) dove N  la posizione nella lista SenseList
numeraLista([],[]):-!.
numeraLista(SenseList,L):-
    numeraElem(SenseList,0,L).
    
%numeraElem(+List,+N,?L).
%numera ogni elemento della lista List tenendo solo le info relative a Word e PartOfSpeech
numeraElem([],_,[]):-!.
numeraElem([sense(_,Word,Pos,_)|Rest],Nprec,[(Word,Pos,N)|K]):-
    N is Nprec + 1,
    numeraElem(Rest,N,K).
    
%bindAdjectives(+ListaNumerata,?CoppieAN).
%CoppieAN  la lista delle coppie (Aggettivo,Nome) in relazione tra loro per vicinanza in ListaNumerata
bindAdjectives(ListaNumerata,CoppieAN):-
    findall((N,Adj),member((Adj,adjective,N),ListaNumerata),Agg),
    findall((N1,Noun),member((Noun,noun,N1),ListaNumerata),Nomi),
    accoppiaAggettivi(Agg,Nomi,CoppieAN).

%accoppiaAggettivi(+Agg,+Nomi,?CoppieAN).
%accoppia gli aggettivi funzione ausiliaria
accoppiaAggettivi([],_,[]):-!.
accoppiaAggettivi([(N,Adj)|R],Nouns,Coppie):-
    findNearestNoun((N,Adj),Nouns,Accoppiato),                                           %associa a Adj i nomi a esso piu vicini in Nouns
    accoppiaAggettivi(R,Nouns,Accoppiati),                                               %ricorre per gli altri aggettivi
    append(Accoppiato,Accoppiati,Coppie).                                                %unisce le liste ottenute ai passi precedenti

%findNearestNoun(+(N,Adj),+Nouns,?Accoppiato).
%Accoppiato  la lista dei nomi piu vicini ad Adj che stanno in Nouns
findNearestNoun((N,Adj),Nouns,Accoppiato):-
    findall((Noun,Diff),(member((I,Noun),Nouns),X is N-I,Diff is abs(X)),L),             %L  formata dalle coppie (Nome, Distanza) -Distanza da Adj-
    findall(Nome,(member((Nome,Dist),L),smaller(Dist,L)),Vicini),                        %Vicini sono i nomi a distanza minima
    findall((Adj,Vicino),member(Vicino,Vicini),Accoppiato).                              %crea le coppie (Adj, Vicino)

%bindAdverbs(+ListaNumerata,?CoppieAV).
%CoppieAV  la lista delle coppie (Avverbio,Verbo) in relazione tra loro per vicinanza in ListaNumerata
bindAdverbs(ListaNumerata,CoppieAV):-
    findall((N,Adv),member((Adv,adverb,N),ListaNumerata),Avverbi),
    findall((N1,Verb),member((Verb,verb,N1),ListaNumerata),Verbi),
    accoppiaAvverbi(Avverbi,Verbi,CoppieAV).

%accoppiaAvverbi(+Avverbi,+Verbi,?CoppieAV)..
%accoppia gli avverbi funzione ausiliaria
accoppiaAvverbi([],_,[]):-!.
accoppiaAvverbi([(N,Adv)|R],Verbs,Coppie):-
    findNearestVerb((N,Adv),Verbs,Accoppiato),                                           %associa a Adj i nomi a esso piu vicini in Nouns
    accoppiaAvverbi(R,Verbs,Accoppiati),                                                 %ricorre per gli altri aggettivi
    append(Accoppiato,Accoppiati,Coppie).                                                %unisce le liste ottenute ai passi precedenti

%findNearestVerb(+(N,Adv),+Verbs,?Accoppiato).
%Accoppiato  la lista dei nomi piu vicini ad Adv che stanno in Verbs
findNearestVerb((N,Adv),Verbs,Accoppiato):-
    findall((Verb,Diff),(member((I,Verb),Verbs),X is N-I,Diff is abs(X)),L),             %L  formata dalle coppie (Nome, Distanza) -Distanza da Adj-
    findall(Verbo,(member((Verbo,Dist),L),smaller(Dist,L)),Vicini),                      %Vicini sono i nomi a distanza minima
    findall((Adv,Vicino),member(Vicino,Vicini),Accoppiato).                              %crea le coppie (Adj, Vicino)

%smaller(+D,+L)
%D  piu piccolo o uguale rispetto a tutti i secondi elementi delle coppie della lista
smaller(_,[]).
smaller(D,[(_,V)|R]):-
    D =< V,
    smaller(D,R).

%createAssociations(+ListaNumerata,?Associations).
%Associations  lista delle triple/coppie formate da nomi e verbi secondo i pattern
createAssociations(ListaNumerata,Assoc):-
    findall((Noun1,Verb,Noun2),(                                         %le terne del tipo (Nome,Verbo,Nome)
        member((Noun1,noun,N1),ListaNumerata),
        member((Verb,verb,N2),ListaNumerata),
        member((Noun2,noun,N3),ListaNumerata),
        N1<N2, N2<N3
    ),L1),
    findall((Noun3,Verb2),(                                              %le coppie del tipo (Nome,Verbo), con Verbo che ancora non sia stato considerato
        member((Noun3,noun,I1),ListaNumerata),
        member((Verb2,verb,I2),ListaNumerata),
        I1<I2,
        not(member((_,Verb2,_),L1))
    ),L2),
    findall((Noun4,Noun5),(                                              %le coppie del tipo (Nome,Verbo), con Verbo che ancora non sia stato considerato
        member((Noun4,noun,M1),ListaNumerata),
        member((Noun5,noun,M2),ListaNumerata),
        M1<M2,
        Dist is M2-M1,
        windowsLength(WL),
        Lung is WL/2,
        Dist < Lung
    ),L3),
    append(L1,L2,L12),
    append(L12,L3,Assoc).