% Author: MB
% Date: 04/12/2009
:- ensure_loaded('wn_hyp.pl').
:- ensure_loaded('wn_s.pl').
:- ensure_loaded('wn_ins.pl').
:- ensure_loaded('wn_g.pl').
:- ensure_loaded('jaroWinkler.pl').
:- ensure_loaded('wn_tools.pl').

%findRoleByGloss(+S,?Role).
%Role  la stringa in relazione di hyperonimia con la stringa associata ad S che meglio ne rappresenta la ascendenza
findRoleByGloss(S,Role):-
     my_s(S,_,Word,_,_,_),!,
     g(S,G),                                                 %recupero il gloss
     atom_codes(G,Chars),                                    %lista di caratteri
     tokenize(Chars,Tokens),                                 %tutte le parole del gloss
     subClassOf(S,SynList),                                  %il Syn degli hiperonym/classi di S
     findall(W,(member(H,SynList),my_s(H,_,W,_,_,_)),L),     %L  la lista degli hyperonim di S
     calcolaDistanze(Tokens,L,Distanze),                     %calcola distanze tra tutti gli elementi di Tokens e quelli di L
     bestRole(Distanze,(Word,0),Role).                       %l'approssimazione migliore

%subClassOf(+S,?SynList).
%SynList  unificato con la lista degli hyperonini diS o , se questa  vuota, con la lista delle classi di cui S  istanza.
subClassOf(S,[H|T]):-
    allHyperonim([S],[H|T]),!.                               %se la lista non  vuota la ritorna
    
subClassOf(S,L):-
    allClasses(S,C),                                         %se non ci sono hyperonimi cerca le classi di cui S  istanza
    allHyperonim(C,H),!,                                     %calcola la lista degli hyperonim delle classi
    append(C,H,L).                                           %e ritorna la concatenazione delle due

%allHyperonim(+L,?SynList).
%SynList  la lista dei synset le cui stringhe sono in relazione di hyperonimia con le stringhe i cui Synset sono in L
allHyperonim([],[]):-!.
allHyperonim([E|T],H):-
    findall(Hyp,hyp(E,Hyp),H1),     %H1 tutti gli hyp di E
    allHyperonim(H1,H2),            %H2 tutti gli hyp degli hyp di E
    allHyperonim(T,H3),             %H3 tutti gli hyp di T
    append(H2,H3,K),
    append(H1,K,K2),
    flatten(K2,H).

%allClasses(+L,?SynList).
%SynList  la lista dei synset le cui stringhe sono in relazione di classe/istanza con le stringhe i cui Synset sono in L
allClasses([],[]):-!.
allClasses(E,Classes):-
    findall(Class,ins(E,Class),Classes).     %I1 tutte le classi di cui E  istanza

%calcolaDistanza(+R,+L,Distanze).
%Distanze  la lista delle triple (Token,Hyp,D), con Token che appartiene a R, Hyp che appartiene a L e D che  la distanza tra loro
calcolaDistanze([],_,[]):-!.
calcolaDistanze([E|Rest],L,Distanze):-
     trovaDistanze(E,L,K),
     calcolaDistanze(Rest,L,Dist),
     append(Dist,K,Distanze).

%trovaDistanze(+E,+L,?List).
%List  la lista delle triple (E,Hip,D), dove Hip appartiene a L e D  la distanze tra E e Hip
trovaDistanze(_,[],[]):-!.
trovaDistanze(E,[H|T],[(E,H,D)|List]):-
     jaroWinklerDistance(E,H,D),!,
     trovaDistanze(E,T,List).

trovaDistanze(E,[_|T],List):-           %aggiunta perch jaroWinkler puo' fallire su parole molto differenti
     trovaDistanze(E,T,List).

%bestRole(+Distanze,+(Word,N),?Role).
%Role  unificato con l'H della tripla (W,H,D) in Distanze la cui D  maggiore di tutte le altre
bestRole([],(H,_),H):-!.
bestRole([(_,H,D)|Rest],(_,N),Role):-
     D > N,!,
     bestRole(Rest,(H,D),Role).

bestRole([(_,_,D)|Rest],(M,N),Role):-
     D =< N,
     bestRole(Rest,(M,N),Role).
