% Author: MB
% Date: 28/11/2009

:- dynamic point/1.
:- dynamic betweenBrackets/1.

%tokenizeToPoint(Testo, ListaParoleFinoAlPunto, Rest)
%unifica ListaParoleFinoAlPunto con la lista delle parole di Testo fino al punto e Rest con il Testo che segue

tokenizeToPoint([], [], []):-!.

tokenizeToPoint([Head|Tail], [Word], R):-
     extractWord([Head|Tail], A, R),                                          %se estrae parola con punto termina
     point(1),!,retractall(point(_)),                                         %se raggiunto il punto taglia
     atom_codes(Word, A).                                                     %crea l'atomo a partire dalla lista di caratteri

tokenizeToPoint([Head|Tail], List, Rest) :-
     extractWord([Head|Tail], [], R),!,                                       %se estrae una lista di caratteri vuota
     tokenizeToPoint(R, List, Rest).                                          %va avanti

tokenizeToPoint([Head|Tail], [Word|List], Rest) :-
     extractWord([Head|Tail], A, R),                                          %se estrae una lista di caratteri non vuota
     atom_codes(Word, A),                                                     %crea l'atomo a partire dalla lista di caratteri
     tokenizeToPoint(R, List, Rest).                                          %va avanti

% extractWord(+List, ?Word, ?Rest)
% Unifica la variabile Word con la prima parola estratta dalla stringa List e la variabile Rest con la lista rimanente.
extractWord([], [], []):-!.
extractWord([Head|Tail], [], Rest):-
     is_open_bracket(Head),!,                                                 %incontrata parentesi aperta
     findCloseBracket(Tail,Phrase,Rest),                                      %Rest  quello che segue la parentesi chiusa
     (
     not(betweenBrackets(Phrase)),                                            %se Phrase non  gi stata asserita
     assert(betweenBrackets(Phrase)),!                                        %Phrase  tutto cio che sta tra parentesi
     ;
     retract(betweenBrackets(Phrase)),                                        %se Phrase  gi stata asserita
     assert(betweenBrackets(Phrase))                                          %la retract e la assert
     ).

extractWord([Head|Tail], [], Tail) :-
     is_punct(Head),!,                                                        % un segno di punteggiatura forte
     assert(point(1)).                                                        %taglia e unifica Word con la lista con un punto e Rest con il resto della stringa

extractWord([Head|Tail], [], Tail) :-
     Head < "A",                                                              %non  un carattere alfabetico
     !.                                                                       %taglia e unifica Word con la lista vuota e Rest con il resto della stringa

extractWord([Head|Tail], [], Tail) :-
     Head > "z",                                                              %non  un carattere alfabetico
     !.                                                                       %taglia e unifica Word con la lista vuota e Rest con il resto della stringa

extractWord([Head|Tail], [], Tail) :-
     Head > "Z",
     Head < "a",                                                              %non  un carattere alfabetico
     !.                                                                       %taglia e unifica Word con la lista vuota e Rest con il resto della stringa

extractWord([Head|Tail], Word, Rest) :-
     Head >= "A",
     Head =< "Z",                                                             % un carattere alfabetico maiuscolo
     extractWord(Tail, P , Rest),                                             %estrae il resto della parola
     append([Head], P , Word),                                                %concatena il carattere con il resto della parola
     !.                                                                       %taglia e unifica Word con la parola trovata e Rest con la stringa senza pi la parola estratta

extractWord([Head|Tail], Word, Rest) :-
     Head >= "a",
     Head =< "z",                                                             % un carattere alfabetico minuscolo
     extractWord(Tail, P , Rest),                                             %estrae il resto della parola
     append([Head], P , Word).                                                %concatena il carattere con il resto della parola e unifica Word con la parola trovata e Rest con la stringa senza pi la parola estratta

%findCloseBracket(+Testo,?Phrase,?Rest)
%unifica Phrase con la stringa racchiusa tra parentesi e Rest con cio' che segue
findCloseBracket([Head|Tail],K,Rest):-
     is_open_bracket(Head),!,                                                 %trovo una parentesi aperta interna
     findCloseBracket(Tail,Phrase,R),                                         %cerco la prima parentesi chiusa
     (
     not(betweenBrackets(Phrase)),                                            %se Phrase non  gi stata asserita
     assert(betweenBrackets(Phrase)),!                                        %Phrase  tutto cio che sta tra parentesi
     ;
     retract(betweenBrackets(Phrase)),                                        %se Phrase  gi stata asserita
     assert(betweenBrackets(Phrase))                                          %la retract e la assert
     ),
     findCloseBracket(R,K,Rest).                                              %cerco la parentesi chiusa esterna

findCloseBracket([Head|Tail],List,Rest):-
     not(is_close_bracket(Head)),!,                                           %se non trova la parentesi chiusa
     findCloseBracket(Tail,K,Rest),                                           %Rest  cio che segue le parentesi
     append([Head],K,List).                                                   %List  cio che si trova tra parentesi

findCloseBracket([Head|Tail],[],Tail):-
     is_close_bracket(Head).                                                  %trovata la parentesi chiusa

% segni di punteggiatura
is_punct(46 /* . */).
is_punct(33 /* ! */).
is_punct(63 /* ? */).
is_punct(59 /* ; */).
is_punct(58 /* : */).
is_open_bracket(40 /* ( */).
is_close_bracket(41 /* ) */).