:- dynamic ass_s/6.

%%%%%%%%%%%%%%%%%%%%%%%%%% Predicates %%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%% get_last(+List, ?Elem)
%%%%%%%%% Unifica la variabile Elem con l'ultimo elemento della lista List.
get_last([Head], Head) :-            %se  la lista formata da un solo elemento
     !.                              %taglia, ha finito
get_last([_|Tail], List) :-          %altrimenti
     get_last(Tail, List).           %va avanti calcolando l'ultimo elemento della sottolista Tail

%%%%%%%%% extract_word(+List, ?Word, ?Rest)
%%%%%%%%% Unifica la variabile Word con la prima parola estratta dalla stringa List in lowercase e la variabile Rest con la lista rimanente.
extract_word([Head|Tail], [], Tail) :-
     Head < "A",                           %non  un carattere alfabetico
     !.                                    %taglia e unifica Word con la lista vuota e Rest con il resto della stringa
extract_word([Head|Tail], [], Tail) :-
     Head > "z",                           %non  un carattere alfabetico
     !.                                    %taglia e unifica Word con la lista vuota e Rest con il resto della stringa
extract_word([Head|Tail], [], Tail) :-
     Head > "Z",
     Head < "a",                           %non  un carattere alfabetico
     !.                                    %taglia e unifica Word con la lista vuota e Rest con il resto della stringa
extract_word([Head|Tail], Word, Rest) :-
     Head >= "A",
     Head =< "Z",                          % un carattere alfabetico maiuscolo
     L is Head+"a"-"A",                    %rende minuscolo
     extract_word(Tail, P, Rest),          %estrae il resto della parola
     append([L], P, Word),                 %concatena il carattere con il resto della parola
     !.                                    %taglia e unifica Word con la parola trovata e Rest con la stringa senza pi la parola estratta
extract_word([Head|Tail], Word, Rest) :-
     Head >= "a",
     Head =< "z",                          % un carattere alfabetico minuscolo
     extract_word(Tail, P, Rest),          %estrae il resto della parola
     append([Head], P, Word).              %concatena il carattere con il resto della parola e unifica Word con la parola trovata e Rest con la stringa senza pi la parola estratta
extract_word([], [], []).

%%%%%%%%% print_list(+List)
%%%%%%%%% Stampa la lista List.
print_list(List) :-
     write('['),                 %stampa parentesi aperta
     print_list_(List),          %stampa lista con predicato ausiliario
     write(']').                 %stampa parentesi chiusa
print_list_([Head]) :-           %un solo elemento
     !,                          %taglia
     write(Head).                %lo stampa
print_list_([Head|Tail]) :-      %pi elementi
     write(Head),                %stampa il primo
     write(', '),                %stampa virgola
     print_list_(Tail).          %stampa il resto della lista
print_list_([]).


%%%%%%%%% recognize_words(+Words, ?List)
%%%%%%%%% Unifica la variabile List con la lista dei singolari e dei verbi al futuro delle parole della lista Words.
recognize_words(Words, List) :-
     write('-- Recognizing words...'),          %stampa informazione
     nl,                                        %nuova linea
     recognize_words_(Words, List).             %predicato ausiliario
recognize_words_([Head|Tail], [Word|List]) :-
     word_recognition(Head, Word),              %trasforma la parola Head nel corrispondente contenuto in WordNet
     recognize_words_(Tail, List).              %va avanti
recognize_words_([], []).


%%%%%%%%% remove_last(+List, ?Result)
%%%%%%%%% Unifica la variabile Result con la lista List senza l'ultimo elemento.
remove_last([H|T], [H|L]) :-
     remove_last(T, L).          %va avanti
remove_last([_], []).            %ultimo elemento, lo scarta
     
%%%%%%%%% tokenize(+Text, ?List)
%%%%%%%%% Unifica la variabile List con la lista di parole estratte dalla lista di caratteri Text.
tokenize([Head|Tail], List) :-
     extract_word([Head|Tail], [], R),          %se estrae una lista di caratteri vuota
     !,                                         %taglia
     tokenize(R, List).                         %va avanti
tokenize([Head|Tail], [Word|List]) :-
     extract_word([Head|Tail], A, R),           %se estrae una lista di caratteri non vuota
     atom_codes(Word, A),                       %crea l'atomo a partire dalla lista di caratteri
     tokenize(R, List).                         %va avanti aggiungendo in testa al risultato la parola estratta
tokenize([], []).

%%%%%%%%% word_recognition(+Word1, ?Word2)
%%%%%%%%% Unifica la variabile Word2 con il singolare o la coniugazione al futuro della parola lowercase Word1.
word_recognition(Word, Word) :-
     my_s(_,_,Word,_,_,_),                %la parola  gi in WordNet
     !.                                %taglia, finito
word_recognition(Word, S) :-           %la parola non  in WordNet, termina in 'es', viene tagliato tale suffisso
     atom_chars(Word, C),              %trasforma l'atomo in una lista di codici di caratteri
     get_last(C, s),                   %l'ultima lettera  's'
     remove_last(C, R1),               %la toglie
     get_last(R1, e),                  %l'ultima lettera  'e'
     remove_last(R1, R2),              %la toglie
     atom_chars(S, R2),                %ricostruisce l'atomo a partire dalla lista modificata
     my_s(_,_,S,_,_,_),                   %la nuova parola  in WordNet?
     !.                                %taglia, la nuova parola  presente in WordNet
word_recognition(Word, S) :-           %il tentativo precedente non ha avuto successo, termina in 's', viene tagliato tale suffisso
     atom_chars(Word, C),              %trasforma l'atomo in una lista di codici di caratteri
     get_last(C, s),                   %l'ultima lettera  's'
     remove_last(C, R1),               %la toglie
     atom_chars(S, R1),                %ricostruisce l'atomo a partire dalla lista modificata
     my_s(_,_,S,_,_,_),                   %la nuova parola  in WordNet?
     !.                                %taglia, la nuova parola  presente in WordNet
word_recognition(Word, S) :-           %il tentativo precedente non ha avuto successo, termina in 'ies', viene tagliato tale suffisso ed aggiunto 'y' in fondo
     atom_chars(Word, C),              %trasforma l'atomo in una lista di codici di caratteri
     get_last(C, s),                   %l'ultima lettera  's'
     remove_last(C, R1),               %la toglie
     get_last(R1, e),                  %l'ultima lettera  'e'
     remove_last(R1, R2),              %la toglie
     get_last(R2, i),                  %l'ultima lettera  'i'
     remove_last(R2, R3),              %la toglie
     append(R3, [y], L),               %lo aggiunge in fondo
     atom_chars(S, L),                 %ricostruisce l'atomo a partire dalla lista modificata
     my_s(_,_,S,_,_,_),                   %la nuova parola  in WordNet?
     !.                                %taglia, la nuova parola  presente in WordNet
word_recognition(Word, S) :-           %il tentativo precedente non ha avuto successo, termina in 'ed', viene tagliato tale suffisso
     atom_chars(Word, C),              %trasforma l'atomo in una lista di codici di caratteri
     get_last(C, d),                   %l'ultima lettera  'd'
     remove_last(C, R1),               %la toglie
     get_last(R1, e),                  %l'ultima lettera  'e'
     remove_last(R1, R2),              %la toglie
     atom_chars(S, R2),                %ricostruisce l'atomo a partire dalla lista modificata
     my_s(_,_,S,_,_,_),                   %la nuova parola  in WordNet?
     !.                                %taglia, la nuova parola  presente in WordNet
word_recognition(Word, S) :-           %il tentativo precedente non ha avuto successo, termina in 'ied', viene tagliato tale suffisso ed aggiunto 'y' in fondo
     atom_chars(Word, C),              %trasforma l'atomo in una lista di codici di caratteri
     get_last(C, d),                   %l'ultima lettera  'd'
     remove_last(C, R1),               %la toglie
     get_last(R1, e),                  %l'ultima lettera  'e'
     remove_last(R1, R2),              %la toglie
     get_last(R2, i),                  %l'ultima lettera  'i'
     remove_last(R2, R3),              %la toglie
     append(R3, [y], L),               %lo aggiunge in fondo
     atom_chars(S, L),                 %ricostruisce l'atomo a partire dalla lista modificata
     my_s(_,_,S,_,_,_),                   %la nuova parola  in WordNet?
     !.                                %taglia, la nuova parola  presente in WordNet
word_recognition(Word, S) :-           %il tentativo precedente non ha avuto successo, termina in 'd', viene tagliato tale suffisso      
     atom_chars(Word, C),              %trasforma l'atomo in una lista di codici di caratteri
     get_last(C, d),                   %l'ultima lettera  'd'
     remove_last(C, R1),               %la toglie
     atom_chars(S, R1),                %ricostruisce l'atomo a partire dalla lista modificata
     my_s(_,_,S,_,_,_),                   %la nuova parola  in WordNet?
     !.                                %taglia, la nuova parola  presente in WordNet
word_recognition(Word, Word).          %non  stato possibile riconoscere la parola, unifica con la parola non modificata
